#include "CircleRobot.h"
#include <libirobot/StateMachine.h>
#include <libirobot/Robot.h>
#include <libirobot/states/DriveState.h>
#include <libirobot/states/FinalState.h>
#include <libirobot/transitions/ValueSensorTransition.h>
#include <libirobot/transitions/FinishedTransition.h>

#define RADIUS 609
#define HALFCIRCLE 3.14*RADIUS

CircleRobot::CircleRobot(char* ttyDevice)
    : Robot(ttyDevice)
{
    setMode(Safe);
    setAutoUpdate( Sensor::All, true );
    m_machine = new StateMachine(this);

    StateMachine* firstHalf = buildHalfCircles(false);
    StateMachine* secondHalf = buildHalfCircles(true);

    DriveState* driveCircle = new DriveState(m_machine, Drive::Medium, -RADIUS);
    FinishedTransition* halfCirclesFinished = new FinishedTransition(driveCircle, firstHalf);

    ValueSensorTransition* circleComplete = new ValueSensorTransition(secondHalf, Sensor::Distance,(ValueSensorTransition::Comparison) (ValueSensorTransition::GreaterThan | ValueSensorTransition::Total | ValueSensorTransition::ResetOnEntry), HALFCIRCLE*2);
    driveCircle->addTransition(circleComplete);

    DriveState* stop = new DriveState(m_machine, 0, 0);
    FinishedTransition* secondHalfFinished = new FinishedTransition(stop, secondHalf);
    secondHalf->addTransition(secondHalfFinished);

    FinishedTransition* pathComplete = new FinishedTransition(stop, secondHalf);
    secondHalf->addTransition(pathComplete);
    firstHalf->addTransition(halfCirclesFinished);

}

StateMachine*
CircleRobot::buildHalfCircles(bool reverse)
{
    int mult = reverse ? -1 : 1;
    StateMachine* halfCircles = new StateMachine(this, m_machine);

    DriveState* driveRight = new DriveState(halfCircles, Drive::Medium, -RADIUS*mult);
    DriveState* driveLeft = new DriveState(halfCircles, Drive::Medium, RADIUS*mult);

    FinalState* halfCirclesComplete = new FinalState(halfCircles);

    ValueSensorTransition* rightComplete = new ValueSensorTransition(driveLeft, Sensor::Distance, (ValueSensorTransition::Comparison) (ValueSensorTransition::GreaterThan | ValueSensorTransition::Total | ValueSensorTransition::ResetOnEntry), HALFCIRCLE);
    ValueSensorTransition* leftComplete = new ValueSensorTransition(halfCirclesComplete, Sensor::Distance, (ValueSensorTransition::Comparison) (ValueSensorTransition::GreaterThan | ValueSensorTransition::Total | ValueSensorTransition::ResetOnEntry), HALFCIRCLE);

    driveRight->addTransition(rightComplete);
    driveLeft->addTransition(leftComplete);

    halfCircles->setStartState(driveRight);

    return halfCircles;
}

void
CircleRobot::loop()
{
	m_machine->loop();
}

