#ifndef DRIVEROBOT_H
#define DRIVEROBOT_H

#include <libirobot/Robot.h>
#include <libirobot/Drive.h>

using namespace iRobot;

/**
 * A robot that plays music, blinks lights, navigates a room, and docks when it
 * gets a chance.
 */
class DriveRobot : public Robot
{
    public:
        typedef enum {
            Left,
            Right,
            Center
        } Side;
        DriveRobot ( const char ttyDevice[] );
        void loop();
        void disconnected();
        
        Side cliffSide();

		/**
		 * Avoids whatever it bumped into or the cliff it almost fell off.
		 */
		void avoid();

		/**
		 * Handles the home base docking logic
		 */
		void dock();

		/**
		 * Determines if the robot has reached a cliff it probably doesn't want
		 * to fall off of
		 */
		bool reachedCliff();
		bool fellOffCliff();
		bool bumped();
		bool bumpedLeft();
		bool bumpedRight();
	private:
		int m_speed;
		uint8_t m_powerColor;
		Drive::TurnDirection m_lastDir;
		bool m_tryingToDock;
		int m_dockTimeout;
		bool m_dockAttempted;
};

#endif
