#include "StateRobot.h"

#include <libirobot/StateMachine.h>
#include <libirobot/states/FinalState.h>
#include <libirobot/states/DriveState.h>
#include <libirobot/transitions/BumpTransition.h>
#include <libirobot/transitions/FinishedTransition.h>
#include <libirobot/transitions/ValueSensorTransition.h>

StateRobot::StateRobot( const char ttyDevice[] )
	: Robot( ttyDevice )
{
    setMode ( Full );
    m_engine = new StateMachine(this);
	DriveState* drive = new DriveState(m_engine, Drive::Medium, Drive::Straight);
    DriveState* turnLeft = new DriveState(m_engine, Drive::Medium, Drive::Left);
    DriveState* turnRight = new DriveState(m_engine, Drive::Medium, Drive::Right);
    StateMachine* turnAround = new StateMachine(this, m_engine);
    
    FinishedTransition* continueDriving = new FinishedTransition(drive, turnAround);
    turnAround->addTransition(continueDriving);
    
    FinalState* turnedAround = new FinalState(turnAround);
    
    DriveState* reverse = new DriveState(turnAround, -Drive::Medium, Drive::Straight);
    DriveState* turn = new DriveState(turnAround, Drive::Medium, Drive::Right);
    
    ValueSensorTransition* stopBackingUp = new ValueSensorTransition(turn, Sensor::Distance, ValueSensorTransition::GreaterThan, -300);
    ValueSensorTransition* stopTurning = new ValueSensorTransition(turnedAround, Sensor::Angle, ValueSensorTransition::LessThan, -10);
    
    reverse->addTransition(stopBackingUp);
    turn->addTransition(stopTurning);

    BumpTransition* bumpLeft = new BumpTransition(turnRight, BumpAndDropSensor::BumpLeft);
    BumpTransition* bumpRight = new BumpTransition(turnLeft, BumpAndDropSensor::BumpRight);
    BumpTransition* noBump = new BumpTransition(drive, BumpAndDropSensor::None);
    
    BumpTransition* turnAroundOnRight = new BumpTransition(turnAround, BumpAndDropSensor::BumpRight);
    BumpTransition* turnAroundOnLeft = new BumpTransition(turnAround, BumpAndDropSensor::BumpLeft);
    
    drive->addTransition(bumpLeft);
    drive->addTransition(bumpRight);
    
    turnLeft->addTransition(turnAroundOnLeft);
    turnLeft->addTransition(noBump);
    turnRight->addTransition(turnAroundOnRight);
    turnRight->addTransition(noBump);
    
    turnAround->setStartState(reverse);
    m_engine->setStartState(drive);
}

StateRobot::~StateRobot()
{
}

void
StateRobot::loop()
{
	m_engine->loop();
}
