#include "WallFollowRobot.h"
#include <libirobot/Robot.h>
#include <libirobot/Drive.h>
#include <libirobot/SerialControl.h>
#include <libirobot/sensors/BumpAndDropSensor.h>
#include <libirobot/sensors/BooleanSensor.h>
#include <libirobot/sensors/ValueSensor.h>

#ifndef __AVR__
#include <iostream>
#endif

using namespace iRobot;

WallFollowRobot::WallFollowRobot ( const char ttyDevice[] )
        : Robot ( ttyDevice ),
		  m_wallFound(false),
		  m_angle(0),
		  m_noWallDistance(0)
{
    setMode ( Full );
	setAutoUpdate( Sensor::All, true );
}

void
WallFollowRobot::loop()
{
	BooleanSensor* wall = static_cast<BooleanSensor*> ( getSensor( Sensor::Wall ) );
    BumpAndDropSensor* s = static_cast<BumpAndDropSensor*> ( getSensor ( Sensor::BumpsAndDrops ) );
    ValueSensor* distance = static_cast<ValueSensor*> ( getSensor ( Sensor::Distance ) );
    uint8_t bumpFlags = s->flags();
    Drive* d = getDrive();
	if (!wall->value()) {
		if (!m_wallFound) {
			m_angle+=1;
			delay(3000);
		}
		d->move(Drive::Medium, -m_angle);
		m_noWallDistance+=distance->value();
		if (m_noWallDistance>1000) {
			m_wallFound = false;
			m_angle = 100;
			m_noWallDistance = -30000;
		}
	} else {
		m_wallFound = true;
		m_angle = 400;
		d->move(Drive::Medium, 390 );
		m_noWallDistance = 0;
	}
	BooleanSensor *left = static_cast<BooleanSensor*>( getSensor(Sensor::CliffLeft));
	BooleanSensor *frontLeft = static_cast<BooleanSensor*>( getSensor(Sensor::CliffFrontLeft));
	BooleanSensor *right= static_cast<BooleanSensor*>( getSensor(Sensor::CliffRight));
	BooleanSensor *frontRight = static_cast<BooleanSensor*>( getSensor(Sensor::CliffFrontRight));
    if ( bumpFlags & BumpAndDropSensor::BumpAny || left->value() || right->value() || frontLeft->value() || frontRight->value() ) {
		d->move( Drive::Medium, Drive::Left );
		m_wallFound = true;
		m_angle = 400;
    }
}

void
WallFollowRobot::disconnected()
{
#ifndef __AVR__
    cout << "Disconnected." << endl;
#endif
    exit ( 0 );
}
