#include "Drive.h"
#include "SerialControl.h"
#include "Robot.h"
#include "Sensor.h"
#include "sensors/ValueSensor.h"
#include <math.h>

using namespace iRobot;

Drive::Drive ( Robot* robot )
        : m_robot ( robot )
{

}

Drive::~Drive()
{
}

void
Drive::turnWheels( int leftWheel, int rightWheel )
{
	SerialControl* ser = m_robot->getSerial();
	ser->write ( SerialControl::DriveDirect );
	ser->write ( ( rightWheel >> 8 ) & 0x00FF );
	ser->write ( rightWheel & 0x00FF );
	ser->write ( ( leftWheel >> 8 ) & 0x00FF );
	ser->write ( leftWheel & 0x00FF );
}

void
Drive::move ( Speed speed, TurnDirection direction )
{
    move ( ( int ) speed, ( int ) direction );
}

void
Drive::move ( int speed, TurnDirection direction )
{
    move ( speed, ( int ) direction );
}

void
Drive::move ( Speed speed, int direction )
{
    move ( ( int ) speed, direction );
}

void
Drive::move ( int speed, int radius )
{
    SerialControl* ser = m_robot->getSerial();
    ser->write ( SerialControl::Drive );
    ser->write ( ( speed >> 8 ) & 0x00FF );
    ser->write ( speed & 0x00FF );
    ser->write ( ( radius>> 8 ) & 0x00FF );
    ser->write ( radius & 0x00FF );

}

int
Drive::waitForRotation( int angle )
{
    ValueSensor* v = static_cast<ValueSensor*>( m_robot->getSensor( Sensor::Angle ) );
    int angleTraveled = 0;
    m_robot->updateSensor( Sensor::Angle );
    do {
        m_robot->updateSensor( Sensor::Angle );
        angleTraveled += fabs(v->value());
    } while (angleTraveled < angle);
    return angleTraveled;
}

int
Drive::waitForDistance( int distance )
{
    ValueSensor* v = static_cast<ValueSensor*>( m_robot->getSensor( Sensor::Distance ) );
    int distanceTraveled = 0;
    m_robot->updateSensor( Sensor::Distance );
    do {
        m_robot->updateSensor( Sensor::Distance );
        distanceTraveled += fabs(v->value());
    } while (distanceTraveled < distance);
    return distanceTraveled;
}
