#include "Robot.h"
#include "Drive.h"
#include "SerialControl.h"

#ifndef __AVR__
#include <unistd.h>
#endif

using namespace std;
using namespace iRobot;

Robot::Robot ( const char ttyDevice[] )
        : m_serial ( new SerialControl ( this, ttyDevice ) ),
        m_drive ( new Drive ( this ) ),
        m_exited ( false ),
        m_running ( false )
{
#ifdef __AVR__
    m_instance = this;
#endif

    m_serial->write ( SerialControl::Start );
    for ( int i = 0;i<42;i++ ) {
        m_sensors[i] = 0;
    }
}

Robot::~Robot()
{
	m_drive->move(0);
	delete m_serial;
	delete m_drive;
}

void
Robot::setMode ( Mode m )
{
    switch ( m ) {
        case Passive:
            m_serial->write ( SerialControl::Start );
            break;
        case Safe:
            m_serial->write ( SerialControl::Safe );
            break;
        case Full:
            m_serial->write ( SerialControl::Full );
            break;
    }
    delay ( 500 );
}

void
Robot::delay ( unsigned int us )
{
#ifdef __AVR__

#else
    usleep ( us );
#endif
}

Drive*
Robot::getDrive() const
{
    return m_drive;
}

SerialControl*
Robot::getSerial() const
{
    return m_serial;
}

int
Robot::run()
{
    if ( m_exited )
        return m_return;
    m_running = true;
    while ( m_running ) {
        updateSensors();
        loop();
    }
    m_exited = true;
    return m_return;
}

void
Robot::exit ( int returnValue )
{
    m_return = returnValue;
    m_running = false;
}

void
Robot::updateSensors()
{
    if (m_autoUpdate[Sensor::All]) {
        m_serial->requestSensorUpdate( Sensor::All );
    } else {
        for(int i = 0; i < NUM_SENSORS;i++) {
            if (m_autoUpdate[i]) {
                m_serial->requestSensorUpdate((Sensor::SensorID)i);
            }
        }
    }
}

void
Robot::updateSensor( Sensor::SensorID sensor )
{
    m_serial->requestSensorUpdate ( sensor );
}

void
Robot::setAutoUpdate( Sensor::SensorID sensor, bool enabled )
{
    m_autoUpdate[(int)sensor] = enabled;
}

Sensor*
Robot::getSensor ( Sensor::SensorID sensor )
{
    Sensor* s = m_sensors[sensor-7];
    if ( s == 0 )
        s = m_sensors[sensor-7] = Sensor::get ( sensor );
    sensorUpdated ( sensor );
    return s;
}

void
Robot::sensorUpdated ( Sensor::SensorID sensor )
{
    Sensor* s = m_sensors[sensor-7];

    int size = Sensor::packetSize( sensor );
    uint8_t sensorData[size];
    for ( int i = 0;i<size;i++ ) {
        sensorData[i] = m_sensorValues[Sensor::packetOffset(sensor)+i];
    }
    if ( s == 0 )
        return;
    s->update ( sensorData );
}

void
Robot::disconnected()
{
    exit ( 0 );
}

void
Robot::setSong(uint8_t songNum, const Song &song)
{
	m_serial->write(SerialControl::SaveSong);
	m_serial->write(songNum);
	m_serial->write(song);
}

void
Robot::playSong(uint8_t songNum)
{
	m_serial->write(SerialControl::PlaySong);
	m_serial->write(songNum);
}

void
Robot::setLEDs(uint8_t leds, uint8_t powerColor, uint8_t powerIntensity)
{
	m_serial->write(SerialControl::LED);
	m_serial->write(leds);
	m_serial->write(powerColor);
	m_serial->write(powerIntensity);
}

#ifdef __AVR__
Robot* Robot::m_instance = 0;
Robot*
Robot::instance()
{
    return m_instance;
}
#endif
