#include "Sensor.h"

#include "sensors/BumpAndDropSensor.h"
#include "sensors/BooleanSensor.h"
#include "sensors/OvercurrentSensor.h"
#include "sensors/IRSensor.h"
#include "sensors/ValueSensor.h"
#include "sensors/ChargeStateSensor.h"
#include "sensors/UnsignedValueSensor.h"
#include "sensors/CargoDigitalInputSensor.h"
#include "sensors/ChargeSourceSensor.h"
#include "sensors/ModeSensor.h"

using namespace iRobot;

Sensor::Sensor()
{
}

Sensor::~Sensor()
{
}

static int packetSizes[] = {
    26, 10, 6, 10, 14, 12, 52, //Groups 0-6
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, //7-16
    1, 1, 2, 2, 1, 2, 2, 1, 2, 2, //17-26
    2, 2, 2, 2, 2, 1, 2, 1, 1, 1, //27-36
    1, 1, 2, 2, 2, 2 //37-42
};

int
Sensor::packetSize ( Sensor::SensorID type )
{
    return packetSizes[type];
}

static int groupOffsets[] = {
    0, 0, 10, 14, 20, 28, 0, //0-6
    0, 1, 2, 3, 4, 5, 6, 7, 8, 9, //7-16
    10, 11, 12, 14, 16, 17, 19, 21, 22, 24, //17-26
    26, 28, 30, 32, 34, 36, 37, 39, 40, 41, // 27-36
    42, 43, 44, 46, 48, 50 //37-42
};

int
Sensor::packetOffset ( Sensor::SensorID type )
{
    return groupOffsets[type];
}


Sensor*
Sensor::get ( SensorID sensor )
{
    switch ( sensor ) {
        case BumpsAndDrops:
            return new BumpAndDropSensor();
        case CliffLeft:
        case CliffFrontLeft:
        case CliffFrontRight:
        case CliffRight:
        case VirtualWall:
        case SongPlaying:
        case Wall:
            return new BooleanSensor();
        case LowDrive:
            return new OvercurrentSensor();
        case IR:
            return new IRSensor();
        case Distance:
        case Angle:
        case Current:
        case Velocity:
        case Radius:
        case RightVelocity:
        case LeftVelocity:
        case BatteryTemperature:
            return new ValueSensor();
        case ChargeState:
            return new ChargeStateSensor();
        case Voltage:
        case BatteryCharge:
        case BatteryCapacity:
        case WallSignal:
        case CliffLeftSignal:
        case CliffFrontLeftSignal:
        case CliffFrontRightSignal:
        case CliffRightSignal:
        case CargoAnalogInput:
        case Song:
        case StreamPackets:
            return new UnsignedValueSensor();
        case CargoDigitalInputs:
            return new CargoDigitalInputSensor();
        case ChargeSources:
            return new ChargeSourceSensor();
        case Mode:
            return new ModeSensor();
    }
}
