#include "State.h"
#include "StateMachine.h"

using namespace iRobot;

State::State(StateMachine* engine)
{
    m_engine = engine;
    if (this != engine)
        engine->addState(this);
}

State::~State()
{
}

void
State::addTransition(StateTransition* transition)
{
    m_transitions.push_back(transition);
}

State*
State::nextState()
{
    TransitionList::iterator it;
    for(it = m_transitions.begin();it!=m_transitions.end();it++) {
        if ((*it)->test())
            return (*it)->nextState();
    }
    return 0;
}

Robot*
State::getRobot() const
{
    return m_engine->getRobot();
}

StateMachine*
State::getMachine() const
{
    return m_engine;
}

void
State::enter()
{
    TransitionList::iterator it;
    for(it = m_transitions.begin();it!=m_transitions.end();it++) {
        (*it)->parentEntered();
    }
}

void
State::leave()
{
    TransitionList::iterator it;
    for(it = m_transitions.begin();it!=m_transitions.end();it++) {
        (*it)->parentLeft();
    }
}
