#include "StateMachine.h"

#include "Debug.h"

#include "states/FinalState.h"

#include <stdlib.h>

using namespace iRobot;

StateMachine::StateMachine(Robot* robot, StateMachine* parent)
    : State((parent == 0) ? this : parent)
{
    m_robot = robot;
    m_current = 0;
    m_start = 0;
    m_finished = true;
}

StateMachine::~StateMachine()
{
    StateList::iterator it;
    for(it = m_states.begin();it!=m_states.end();it++) {
        delete *it;
    }
}

bool
StateMachine::finished() const
{
    return m_finished;
}

void
StateMachine::setFinished()
{
    m_finished = true;
}

Robot*
StateMachine::getRobot() const
{
   return m_robot;
}

void
StateMachine::addState(State* state)
{
    m_states.push_back(state);
    if (m_current == 0)
        m_current = state;
	if (m_start == 0)
		m_start = state;
}

void
StateMachine::setStartState(State* state)
{
    m_start = state;
}

void
StateMachine::loop()
{
    if (m_finished) {
        m_current = m_start;
        m_current->enter();
        start();
    }
    m_current->loop();
    State* next = m_current->nextState();
    if (next != 0) {
        m_current->leave();
        m_current = next;
        m_current->enter();
    }
}

void
StateMachine::start()
{
    m_finished = false;
}

