#ifndef IRSENSOR_H
#define IRSENSOR_H

#include <libirobot/Sensor.h>

namespace iRobot
{

/**
 * The iRobot's IR reciever, located in the 'beacon' on the top of the bumper.
 */
class IRSensor : public Sensor
{
    public:
		/**
		 * Common IR values for iRobot hardware
		 */
		typedef enum {
			/**
			 * 'Left' on the remote control
			 */
			Left = 129,
			/**
			 * 'Forward' on the remote control
			 */
			Forward = 130,
			/**
			 * 'Right' on the remote control
			 */
			Right = 131,
			/**
			 * 'Spot' on the remote control
			 */
			Spot = 132,
			/**
			 * 'Max' on the remote control
			 */
			Max = 133,
			/**
			 * 'Small' on the remote control
			 */
			Small = 134,
			/**
			 * 'Medium' on the remote control
			 */
			Medium = 135,
			/**
			 * 'Large' on the remote control
			 */
			Large = 136,
			/**
			 * Pause button on the remote control
			 */
			Pause = 137,
			/**
			 * Power button on the remote control
			 */
			Power = 138,
			/**
			 * Forward and left pushed at the same time on the remote control
			 */
			ArcForwardLeft = 139,
			/**
			 * Forward and right pushed at the same time on the remote control
			 */
			ArcForwardRight = 140,
			/**
			 * 'Stop' on the remote control
			 */
			Stop = 141,
			/**
			 * Used on the scheduling remote control
			 */
			SendAll = 142,
			/**
			 * Used on the scheduling remote control
			 */
			SeekDock = 143,
			/**
			 * Reserved for future iRobot use
			 */
			Reserved = 240,
			/**
			 * Red buoy on the home base
			 */
			RedBuoy = 248,
			/**
			 * Green buoy on the home base
			 */
			GreenBuoy = 244,
			/**
			 * Force field on the virtual walls and home base
			 */
			ForceField = 242,
			/**
			 * Value for when both buoys are detected
			 */
			RedAndGreenBuoys = 252,
			/**
			 * Value for when both the red buoy and the home base's force field
			 * are detected.
			 */
			RedBuoyAndForceField = 250,
			/**
			 * Value for when both the green buoy and the home base's force
			 * field are detected.
			 */
			GreenBuoyAndForceField = 246,
			/**
			 * Value for when all three IR emitters (Both buoys and the force
			 * field) on the home base are detected.
			 */
			RedAndGreenBuoysAndForceField = 254
		} Values;
		/**
		 * The value detected on the IR sensor
		 */
        unsigned char value() const;
    protected:
        void update ( uint8_t data[] );
    private:
        char m_value;
};

}

#endif
